*MORAL HAZARD 3 Actions 3 Outcome
*Model 4c
*MLRC no & fails, CDFC yes
*Itoh,"A Course in Contract"4.2.2 p.161  IP'
Option limcol=0, limrow=0;

* Definition of Set
Set     x       project outcome /failure, moderate, success/
        y       effort level    /low, middle, high/
        t       loop index      /1,2,3/;
Alias (y,v);
* Definition of Parameters
Table p(x,y)    probability of outcome by effort level
                low     middle  high
failure         0.7      0.2    0.1
moderate        0.2      0.6    0.2
success         0.1      0.2   0.7
;

Parameter       d(y)    disutility by efforts
        /low    2
        middle  2.1
        high    3/
;
Parameter       r(x) project's interest
        /failure 13
        moderate 29
        success  45/
;
Parameter dummy(y)      dummy to enable and disable equations;
dummy(y)=0;
* Definition of Primal Variables
Positive Variable
                w(x)    entrep's income
;

Variable        U       investor's expected utility
;
Equation
                obj     utility function
                pc(y)   participation constraint
                ic(v,y) incentive compatibility constraint
;

* Defining and Solving the Model
*more active efforts
obj..           U =e= sum((x,y), dummy(y)*p(x,y)*(r(x) - w(x)));
pc(y)..         (sum(x,p(x,y)*sqrt(w(x)))-d(y))*dummy(y) =g= 0;
ic(v,y)..       ((sum(x,p(x,v)*sqrt(w(x)))-d(v))
                -(sum(x,p(x,y)*sqrt(w(x)))-d(y)))*dummy(v) =g= 0;

* Setting Lower Bounds on Variables to Avoid Division by Zero
w.lo(x)=0.0001;

Model MH004FB  First Best Model  /obj,pc   /;
Model MH004SB  Second Best Model /obj,pc,ic/;

Parameter
        w_FB(y,x)  payments
        u_e_FB(y,x) entrep's FB utility
        C_FB(y)    investor's FB expected payment
        B_FB(y)    investor's FB expected revenue
        Net_FB(y)  investor's FB expected utility

        w_SB(y,x)  payments
        u_e_SB(y,x) entrep's utility
        C_SB(y)    investor's expected payment
        B_SB(y)    investor's expected revenue
        Net_SB(y)  investor's expected utility
;

Loop(t,
dummy(y)=1$(ord(t) eq ord(y));

Solve MH004FB maximizing U using NLP;
w_FB(y,x)       $(ord(t) eq ord(y))=w.l(x);
u_e_FB(y,x)     $(ord(t) eq ord(y))=sqrt(w.l(x));
C_FB(y)         $(ord(t) eq ord(y))=sum(x, p(x,y)*w.l(x));
B_FB(y)         $(ord(t) eq ord(y))=sum(x, p(x,y)*r(x));
Net_FB(y)       $(ord(t) eq ord(y))=B_FB(y)-C_FB(y);

Solve MH004SB maximizing U using NLP;
w_SB(y,x)       $(ord(t) eq ord(y))=w.l(x);
u_e_SB(y,x)     $(ord(t) eq ord(y))=sqrt(w.l(x));
C_SB(y)         $(ord(t) eq ord(y))=sum(x, p(x,y)*w.l(x));
B_SB(y)         $(ord(t) eq ord(y))=sum(x, p(x,y)*r(x));
Net_SB(y)       $(ord(t) eq ord(y))=B_SB(y)-C_SB(y);

);

File sol /solution.csv/;
put sol;
sol.pc=5;
sol.pw=32767;


* Generating a solution summary file in CSV
Loop(y,
put "Effort Level:" y.tl /;
put "First Best" "w: payment" "u: entrep's utility" "U: investor's utility" "B: investor's benefit" "C: investor's cost" /;
put "" "" ""  Net_FB(y):10:3 B_FB(y):10:3 C_FB(y):10:3/;
Loop(x, put x.tl w_FB(y,x):10:3 u_e_FB(y,x):10:3 /;);
put /;

put "Second Best" "w: payment" "u: entrep's utility" "U: investor's utility" "B: investor's benefit" "C: investor's cost" /;

put "" "" ""  Net_SB(y):10:3 B_SB(y):10:3 C_SB(y):10:3 /;
Loop(x, put x.tl w_SB(y,x):10:3 u_e_SB(y,x):10:3 /;);
put //;

);

* End of Model
